//+------------------------------------------------------------------+
//|                                                       CPivot.mqh |
//|                                    Copyright 2017, Erwin Beckers |
//|                                              www.erwinbeckers.nl |
//+------------------------------------------------------------------+
#property copyright "Copyright 2017, Erwin Beckers"
#property link      "www.erwinbeckers.nl"
#property strict


enum PivotZone
{
   PivotS38,
   PivotS61,
   PivotS78,
   PivotS100,
   PivotS138,
   PivotS200,
   PivotR38,
   PivotR61,
   PivotR78,
   PivotR100,
   PivotR138,
   PivotR200,
};


// inputs for pivot filter
//extern string     __pivot              = " ------- pivot settings ------------";
//extern int        lookback             = 10;

//--------------------------------------------------------------------
class CPivot
{
private:
   string _symbol;
   ENUM_TIMEFRAMES _pivotTimeFrame;
   double _r38;
   double _r61;
   double _r78;
   double _r100;
   double _r138;
   double _r161;
   double _r200;
   double _s38;
   double _s61;
   double _s78;
   double _s100;
   double _s138;
   double _s161;
   double _s200;
   
   
public:   
   
   //--------------------------------------------------------------------
   CPivot(string symbol,ENUM_TIMEFRAMES pivotTimeFrame)
   {   
      _symbol = symbol;
      _pivotTimeFrame = pivotTimeFrame;
   }

   //+------------------------------------------------------------------+
   void Calculate(bool forceRefresh = false)
   {  
      //double close=iClose(_symbol,TradingTimeFrame,1);
      datetime WeekDate    =  Time[0];
      int CurrentPivotTF = _pivotTimeFrame;
      int    WeeklyBar     =  iBarShift(_symbol, CurrentPivotTF, WeekDate, false)+1;
      //WeeklyBar = 0;
      
      double PreviousHigh  =  iHigh(_symbol, CurrentPivotTF, WeeklyBar);
      double PreviousLow   =  iLow(_symbol, CurrentPivotTF, WeeklyBar);
      double PreviousClose =  iClose(_symbol, CurrentPivotTF, WeeklyBar);

      // ************************************************************************
      //    Calculate Pivot lines and map into indicator buffers.
      // ************************************************************************
      double P  =  (PreviousHigh+PreviousLow+PreviousClose)/3;
      _r38 = P + ((PreviousHigh-PreviousLow) * 0.382);
      _s38 = P - ((PreviousHigh-PreviousLow) * 0.382);
      _r61 = P + ((PreviousHigh-PreviousLow) * 0.618);
      _s61 = P - ((PreviousHigh-PreviousLow) * 0.618);
      _r100 = P + ((PreviousHigh-PreviousLow) * 1.000);
      _s100 = P - ((PreviousHigh-PreviousLow) * 1.000);
      _r78 = P + ((PreviousHigh-PreviousLow) * 0.786);
      _s78 = P - ((PreviousHigh-PreviousLow) * 0.786);
      _r138 = P + ((PreviousHigh-PreviousLow) * 1.382);
      _s138 = P - ((PreviousHigh-PreviousLow) * 1.382);
      _r161 = P + ((PreviousHigh-PreviousLow) * 1.618);
      _s161 = P - ((PreviousHigh-PreviousLow) * 1.618);
      _r200 = P + ((PreviousHigh-PreviousLow) * 2.000);
      _s200 = P - ((PreviousHigh-PreviousLow) * 2.000);
   }
   
   //+------------------------------------------------------------------+
   bool IsAtSupport(double price, double pips, PivotZone zone,bool checkAbove=true)
   {
      Calculate();
      double points   = MarketInfo(_symbol, MODE_POINT);
      double digits   = MarketInfo(_symbol, MODE_DIGITS);
      double mult     = (digits==3 || digits==5) ? 10.0:1;
      double priceRange = pips * mult * points;
      if (DoesSupportLevelExists(price, priceRange, zone,checkAbove)) return true;
      return false;
   }
   
   //+------------------------------------------------------------------+
   bool IsAtResistance(double price, double pips,  PivotZone zone,bool checkBelow=true)
   {
      Calculate();
      double points   = MarketInfo(_symbol, MODE_POINT);
      double digits   = MarketInfo(_symbol, MODE_DIGITS);
      double mult     = (digits==3 || digits==5) ? 10.0:1;
      double priceRange = pips * mult * points;
      
      if (DoesResistanceLevelExists(price, priceRange, zone,checkBelow)) return true;
      return false;
   }
   
   //+------------------------------------------------------------------+
   double GetZonePrice(PivotZone zone)
   {
      if (zone == PivotS38) return (_s38);
      else if (zone == PivotS61) return (_s61);
      else if (zone == PivotS78) return (_s78);
      else if (zone == PivotS100) return (_s100);
      else if (zone == PivotS138) return (_s138);
      else if (zone == PivotS200) return (_s200);
      else if (zone == PivotR38) return (_r38);
      else if (zone == PivotR61) return (_r61);
      else if (zone == PivotR100) return (_r100);
      else if (zone == PivotR138) return (_r138);
      else if (zone == PivotR200) return (_r200);
      
      return (-1);
      
   }
   
   //+------------------------------------------------------------------+
   bool DoesSupportLevelExists(double price, double priceRange, PivotZone zone,bool checkAbove)
   { 
         if (!checkAbove || price >= GetZonePrice(zone))
         {
            double diff = MathAbs(price - GetZonePrice(zone));
            if (diff <= priceRange) 
            {
               //Print("DoesSupportLevelExists " + price + " zone " + zone);
               //srLevelFound = _lines[i].Price;
               return true;
            }
         }
      return false;
   }
   
   //+------------------------------------------------------------------+
   bool DoesResistanceLevelExists(double price, double priceRange, PivotZone zone,bool checkBelow)
   { 
         double zoneprice = GetZonePrice(zone);
         if (!checkBelow || price <= zoneprice)
         {
            double diff = MathAbs(price - zoneprice);
            if (diff <= priceRange) 
            {
               //Print("DoesResistanceLevelExists " + price + " zone " + zone + " zoneprice " + zoneprice + " pricerange " + priceRange + " diff " + diff);
               //srLevelFound = _lines[i].Price;
               return true;
            }
         }
      return false;
   }
   
   
   
   //--------------------------------------------------------------------
   void Refresh(string symbol)
   {
      
         
   }
};
